Class: Riffer::StreamEvents::ToolCallDelta

Inherits:
Base
  • Object
show all
Defined in:
lib/riffer/stream_events/tool_call_delta.rb

Overview

Riffer::StreamEvents::ToolCallDelta represents an incremental tool call chunk during streaming.

Emitted when the LLM is building a tool call, containing partial argument data.

Instance Attribute Summary collapse

Attributes inherited from Base

#role

Instance Method Summary collapse

Constructor Details

#initialize(item_id:, arguments_delta:, name: nil, role: "assistant") ⇒ ToolCallDelta

Creates a new tool call delta event

Parameters:

  • item_id (String)

    the tool call item identifier

  • name (String, nil) (defaults to: nil)

    the tool name (may only be present in first delta)

  • arguments_delta (String)

    the incremental arguments JSON fragment

  • role (String) (defaults to: "assistant")

    the message role (defaults to “assistant”)



16
17
18
19
20
21
# File 'lib/riffer/stream_events/tool_call_delta.rb', line 16

def initialize(item_id:, arguments_delta:, name: nil, role: "assistant")
  super(role: role)
  @item_id = item_id
  @name = name
  @arguments_delta = arguments_delta
end

Instance Attribute Details

#arguments_deltaObject (readonly)



9
10
11
# File 'lib/riffer/stream_events/tool_call_delta.rb', line 9

def arguments_delta
  @arguments_delta
end

#item_idObject (readonly)



9
10
11
# File 'lib/riffer/stream_events/tool_call_delta.rb', line 9

def item_id
  @item_id
end

#nameObject (readonly)



9
10
11
# File 'lib/riffer/stream_events/tool_call_delta.rb', line 9

def name
  @name
end

Instance Method Details

#to_hHash

Converts the event to a hash

Returns:

  • (Hash)

    the event data



25
26
27
# File 'lib/riffer/stream_events/tool_call_delta.rb', line 25

def to_h
  {role: @role, item_id: @item_id, name: @name, arguments_delta: @arguments_delta}.compact
end